package main

import (
	"flag"
	"fmt"
	"io"
	"log"
	"net/http"
	"os"
	"time"
)

const usage = `Usage: https_visitor_counter [options] [HOST]:PORT

A simple HTTPS server that returns the number of visitors to /count.

positional arguments
  [HOST]:PORT

  A "host:port" address that the target listens on for HTTPS requests.

options:
  -cert TLS_CERT
    The TLS certificate file for the server.

    Default: server.crt

  -key TLS_PRIVATE_KEY
    The TLS private key for the server.

    Default: server.key

  -help
    Display this usage statement and exit.

examples:
$ ./https_visitor_counter -cert assets/server.crt -key assets/server.key 127.0.0.1:8054
`

type Options struct {
	// positional
	hostport string
	// options
	certFile string
	keyFile  string
}

func printUsage() {
	fmt.Fprintf(os.Stderr, "%s", usage)
}

func parseOptions() *Options {
	options := Options{}

	flag.Usage = printUsage
	flag.StringVar(&options.certFile, "cert", "server.crt", "")
	flag.StringVar(&options.keyFile, "key", "server.key", "")

	flag.Parse()

	if flag.NArg() != 1 {
		log.Fatalf("expected one positional argument but got %d", flag.NArg())
	}

	options.hostport = flag.Arg(0)

	return &options
}

const countPageTemplate = `<!DOCTYPE html>
<html lang="en">
<head>
<title>Visitor Counter</title>
<meta charset="utf-8">
</head>
<body>
<h1>Welcome Visitor #%d</h1>
</body>
</html>
`

type CountHandler struct {
	numVisitors int
}

func newCountHandler() *CountHandler {
	return &CountHandler{}
}

func (h *CountHandler) ServeHTTP(w http.ResponseWriter, r *http.Request) {
	h.numVisitors += 1
	w.Header().Set("Content-Type", "text/html")
	w.WriteHeader(http.StatusOK)
	io.WriteString(w, fmt.Sprintf(countPageTemplate, h.numVisitors))
}

func newServer(hostport string) *http.Server {
	h := newCountHandler()
	mux := http.NewServeMux()
	mux.Handle("/count", h)
	s := &http.Server{
		Addr:         hostport,
		ReadTimeout:  30 * time.Second,
		WriteTimeout: 30 * time.Second,
		IdleTimeout:  30 * time.Second,
		Handler:      mux,
	}

	return s
}

func main() {
	options := parseOptions()
	s := newServer(options.hostport)
	err := s.ListenAndServeTLS(options.certFile, options.keyFile)
	if err != http.ErrServerClosed {
		log.Fatal(err)
	}
}
